@@ -1,7 +1,22 @@
 import numpy as np
 from ndn4sid.constants import *
 
-## All Linear algebra routines
+def get_random_matrix_with_rank_r(n,r=0):
+    """
+    A function to generate random matrices with a predefined rank r.
+    The rank must be smaller than the provided size of the matrix.
+    Parameters:
+        n(int): The size of the returned matrix
+        r(int): The rank 
+    Returns:
+        A: A 2D numpy array of size n times n, with rank r.    
+    """
+    A_full_rank = np.random.randn(n,n)
+    U,s,V = np.linalg.svd(A_full_rank)
+    s[r:] = 0
+    A = U@np.diag(s)@V
+    return A
+    
 
 
 def get_random_permutation(n):
@@ -10,16 +25,16 @@ def get_random_permutation(n):
     Parameters:
         n(int): size of the permutation matrix
     Returns:
-        M: 2D numpyarray a random permutation
-        Minv: 2D numpy array the inverse permutation of specified by M
+        M: 2D numpy array a random permutation
+        M_inv: 2D numpy array the inverse permutation of specified by M
     """
     i = np.random.randint(n)
     j = np.mod(i+np.random.randint(1,n),n)
     M = np.eye(n,dtype=np.int)
-    Minv = np.eye(n,dtype=np.int)
+    M_inv = np.eye(n,dtype=np.int)
     M[i,j] = 1
-    Minv[i,j] = -1
-    return M,Minv
+    M_inv[i,j] = -1
+    return M,M_inv
 
 
 def get_right_null_space(M, r=0):
@@ -32,8 +47,8 @@ def get_right_null_space(M, r=0):
         right_null_space: A 2D numpy array, which is the right null space of M.
     """
     r = np.linalg.matrix_rank(M) if r == 0 else r
-    U, _, _ = np.linalg.svd(M, full_matrices=True)
-    right_null_space = U[:, r:]
+    _, _, V = np.linalg.svd(M, full_matrices=True)
+    right_null_space = V[r:, :].T
     return right_null_space
 
 
@@ -78,7 +93,7 @@ def get_difference_column_space(T, T_0):
     Returns:
         T_1: A 2D numpy array, which is the difference between T and T_0
     """
-    T_1 = get_intersection(T, get_right_null_space(T_0))
+    T_1 = get_intersection(T, get_complement_columns(T_0))
     return T_1
 
 
@@ -96,19 +111,23 @@ def concat(A, B, axis=0):
     return AB
 
 
-def get_intersection(A, B):
+def get_intersection(A, B, dim_intersection=None):
     """
     Calculates the intersection of of two column spaces. This space is V = {x | ∃v,∃w, Av = x = Bw}
     Parameters:
         A: A 2D numpy array
         B: A 2D numpy array
+        dim_intersection: An optional estimation of the dimension of the intersection.
     Returns:
         intersection: The intersection of the column spaces of A and B.
     """
     complement_A = get_complement_columns(A)
     complement_B = get_complement_columns(B)
     concat_comp_A_comp_B = concat(complement_A, complement_B, axis=1)
-    intersection = get_complement_columns(concat_comp_A_comp_B)
+    if dim_intersection == None:
+        intersection = get_complement_columns(concat_comp_A_comp_B)
+    else:
+        intersection = get_complement_columns(concat_comp_A_comp_B,A.shape[0]-dim_intersection)
     return intersection
 
 
@@ -131,13 +150,13 @@ def get_projector(A, r=0):
 
 def project_A_on_B(A, B, r=0):
     """
-    Projectes the columns of A on to the column space of B. 
+    Projects the columns of A on to the column space of B. 
     The rank of B can be be provided. If no rank is provided, the rank is estimated. 
     Parameters:
         A: A 2D numpy array
         B: A 2D numpy array
     Returns:
-        projection: the projeced space from the columns of A onto the columns of B.
+        projection: the projected space from the columns of A onto the columns of B.
     """
     projection = get_projector(B, r) @ A
     return projection
@@ -163,7 +182,7 @@ def lie(A, B):
         A: 2D numpy array
         B: 2D numpy array
     Returns:
-        commutator: 2D numpy array representing the comutator of A and B.
+        commutator: 2D numpy array representing the commutator of A and B.
     """
     commutator = A @ B - B @ A
     return commutator
\ No newline at end of file
